/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import java.io.IOException;

import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Workspace;

import javax.servlet.jsp.JspException;

import com.squeakysand.jcr.Namespace;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class ForEachNamespaceTag extends JcrSessionAwareSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(ForEachNamespaceTag.class);
    private NamespaceRegistry namespaceRegistry;
    private String namespaceVar;
    private Workspace workspace;

    @Override
    public void doTag() throws JspException, IOException {
        try {
            NamespaceRegistry determinedNamespaceRegistry = determineNamespaceRegistry();
            if (determinedNamespaceRegistry != null) {
                String[] uris = determinedNamespaceRegistry.getURIs();
                if (uris != null) {
                    String determinedNamespaceVar = determineNamespaceVar();
                    for (String uri : uris) {
                        String prefix = determinedNamespaceRegistry.getPrefix(uri);
                        getJspContext().setAttribute(determinedNamespaceVar, new Namespace(prefix, uri));
                        getJspBody().invoke(null);
                    }
                }
            }
        } catch (RepositoryException e) {
            throw new JspException(e);
        }
    }

    public NamespaceRegistry getNamespaceRegistry() {
        return namespaceRegistry;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setNamespaceRegistry(NamespaceRegistry namespaceRegistry) {
        this.namespaceRegistry = namespaceRegistry;
    }

    public String getNamespaceVar() {
        return namespaceVar;
    }

    @JspTagAttribute
    public void setNamespaceVar(String namespaceVar) {
        this.namespaceVar = namespaceVar;
    }

    public Workspace getWorkspace() {
        return workspace;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setWorkspace(Workspace workspace) {
        this.workspace = workspace;
    }

    private NamespaceRegistry determineNamespaceRegistry() throws RepositoryException {
        NamespaceRegistry result = null;
        if (namespaceRegistry != null) {
            result = namespaceRegistry;
        } else {
            Workspace determinedWorkspace = determineWorkspace();
            if (determinedWorkspace != null) {
                result = determinedWorkspace.getNamespaceRegistry();
            }
        }
        return result;
    }

    private Workspace determineWorkspace() {
        Workspace result = null;
        if (workspace != null) {
            result = workspace;
        } else {
            Session session = determineSession();
            if (session != null) {
                result = session.getWorkspace();
            }
        }
        return result;
    }

    private String determineNamespaceVar() {
        String result = namespaceVar;
        if (result == null) {
            result = AttributeNames.CURRENT_NAMESPACE;
        }
        return result;
    }
}
