/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import javax.jcr.Session;

import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class JcrSessionAwareSimpleTagSupport extends EnhancedSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(JcrSessionAwareSimpleTagSupport.class);

    private Session jcrSession;

    public Session getJcrSession() {
        return jcrSession;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setJcrSession(Session session) {
        this.jcrSession = session;
    }

    protected Session determineSession() {
        Session result = null;
        if (jcrSession != null) {
            result = jcrSession;
        } else {
            try {
                result = (Session)  getJspContext().findAttribute(AttributeNames.CURRENT_SESSION);
            } catch (ClassCastException e) {
                LOG.error(e.getMessage(), e);
            }
        }
        return result;
    }
}
