/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.QueryResult;

import javax.servlet.jsp.JspException;

import com.squeakysand.jcr.query.QueryRunner;
import com.squeakysand.jcr.query.QueryType;
import com.squeakysand.jsp.tagext.annotations.JspTag;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@JspTag
public class QueryTag extends JcrSessionAwareSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(QueryTag.class);

    private String query;
    private QueryType queryType;
    private String resultVar;
//    private Session session;

    @Override
    public void doTag() throws JspException {
        if (LOG.isTraceEnabled()) {
            LOG.trace("called");
        }
        QueryType determinedQueryType = determineQueryType();
        Session determinedSession = determineSession();
        QueryResult queryResult = null;
        try {
            queryResult = QueryRunner.executeQuery(query, determinedQueryType, determinedSession);
        } catch (InvalidQueryException e) {
            throw new JspException(e);
        } catch (RepositoryException e) {
            throw new JspException(e);
        }
        if (LOG.isDebugEnabled()) {
            LOG.debug("setting var %s to %s", getResultVar(), queryResult);
        }
        getJspContext().setAttribute(getResultVar(), queryResult);
    }

    public String getQuery() {
        return query;
    }

    @JspTagAttribute(
        required = true,
        rtexprvalue = true
    )
    public void setQuery(String query) {
        this.query = query;
    }

    public String getResultVar() {
        return resultVar;
    }

    @JspTagVariable(
        variableClass = QueryResult.class,
        scope = VariableScope.AT_END
    )
    @JspTagAttribute(required = true)
    public void setResultVar(String resultVar) {
        this.resultVar = resultVar;
    }

    public QueryType getQueryType() {
        return queryType;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setQueryType(QueryType queryType) {
        this.queryType = queryType;
    }

    private QueryType determineQueryType() {
        // default to SQL2
        QueryType result = QueryType.SQL;
        if (queryType != null) {
            result = queryType;
        }
        return result;
    }
}
